@@ -6,11 +6,16 @@ use base qw/Data::Visitor/;
 use strict;
 use warnings;
 
-__PACKAGE__->mk_accessors( "callbacks" );
+__PACKAGE__->mk_accessors( qw/callbacks ignore_return_values/ );
 
 sub new {
 	my ( $class, %callbacks ) = @_;
 
+	my $ignore_ret = 0;
+	if	( exists $callbacks{ignore_return_values} ) {
+		$ignore_ret = delete $callbacks{ignore_return_values};
+	}
+
 	my $self = $class->SUPER::new();
 
 	$self->callbacks( \%callbacks );
@@ -20,6 +25,7 @@ sub new {
 
 sub visit {
 	my ( $self, $data ) = @_;
+	local *_ = \$_[1]; # alias $_
 	$self->SUPER::visit( $self->callback( visit => $data ) );
 }
 
@@ -35,21 +41,29 @@ sub visit_object {
 	$self->callback( object => $data );
 }
 
-sub visit_hash {
-	my ( $self, $data ) = @_;
-	$self->SUPER::visit_hash( $self->callback( hash => $data ) );
-}
-
-sub visit_array {
-	my ( $self, $data ) = @_;
-	$self->SUPER::visit_array( $self->callback( array => $data ) );
+BEGIN {
+	foreach my $reftype ( qw/array hash glob scalar/ ) {
+		no strict 'refs';
+		*{"visit_$reftype"} = eval '
+			sub {
+				my ( $self, $data ) = @_;
+				my $new_data = $self->callback( '.$reftype.' => $data );
+				if ( ref $data eq ref $new_data ) {
+					$self->SUPER::visit_'.$reftype.'( $new_data );
+				} else {
+					$self->SUPER::visit( $new_data );
+				}
+			}
+		' || die $@;
+	}
 }
 
 sub callback {
 	my ( $self, $name, $data ) = @_;
 
 	if ( my $code = $self->callbacks->{$name} ) {
-		return $code->( $self, $data );
+		my $ret = $code->( $self, $data );
+		return $self->ignore_return_values ? $data : $ret ;
 	} else {
 		return $data;
 	}
@@ -85,9 +99,23 @@ needing to subclass yourself.
 
 =over 4
 
-=item new %callbacks
+=item new %opts, %callbacks
 
+Construct a new visitor.
 
+The options supported are:
+
+=over 4
+
+=item ignore_return_values
+
+When this is true (off by default) the return values from the callbacks are
+ignored, thus disabling the fmapping behavior as documented in
+L<Data::Validator>.
+
+This is useful when you want to modify $_ directly
+
+=back
 
 =back
 
@@ -100,11 +128,14 @@ The callback is in the form:
 	sub {
 		my ( $visitor, $data ) = @_;
 
-		# ...
+		# or you can use $_, it's aliased
 
 		return $data; # or modified data
 	}
 
+Within the callback $_ is aliased to the data, and this is also passed in the
+parameter list.
+
 =over 4
 
 =item visit
@@ -137,6 +168,16 @@ Called for hash references.
 
 =back
 
+=head1 AUTHOR
+
+Yuval Kogman <nothingmuch@woobling.org>
+
+=head1 COPYRIGHT & LICENSE
+
+	Copyright (c) 2006 Yuval Kogman. All rights reserved
+	This program is free software; you can redistribute
+	it and/or modify it under the same terms as Perl itself.
+
 =cut
 
 
